/*
+----------------------------------------------------------------------+
| PHP Version 5                                                        |
+----------------------------------------------------------------------+
| Copyright (c) 1997-2004 The PHP Group                                |
+----------------------------------------------------------------------+
| This source file is subject to version 3.0 of the PHP license,       |
| that is bundled with this package in the file LICENSE, and is        |
| available through the world-wide-web at the following url:           |
| http://www.php.net/license/3_0.txt.                                  |
| If you did not receive a copy of the PHP license and are unable to   |
| obtain it through the world-wide-web, please send a note to          |
| license@php.net so we can mail you a copy immediately.               |
+----------------------------------------------------------------------+
| Author: Markus Nix <mnix@docuverse.de>                               |
+----------------------------------------------------------------------+
*/

/* $Id: huffman.c,v 1.3 2004/08/26 11:10:36 docuverse_de Exp $ */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "php.h"
#include "php_ini.h"
#include "ext/standard/info.h"
#include "php_huffman.h"
#include "huffalgo.h"


/* True global resources - no need for thread safety here */
static int le_huffman;

/* {{{ huffman_functions[]
 */
function_entry huffman_functions[] = {
    PHP_FE(huffman_encode,    NULL)
    PHP_FE(huffman_decode,    NULL)
    PHP_FE(huffman_encode_file,    NULL)
    PHP_FE(huffman_decode_file,    NULL)
    {NULL, NULL, NULL}
};
/* }}} */

/* {{{ huffman_module_entry
 */
zend_module_entry huffman_module_entry = {
#if ZEND_MODULE_API_NO >= 20010901
    STANDARD_MODULE_HEADER,
#endif
    "huffman",
    huffman_functions,
    PHP_MINIT(huffman),
    NULL,
    NULL,
    NULL,
    PHP_MINFO(huffman),
#if ZEND_MODULE_API_NO >= 20010901
    "0.2", /* Replace with version number for your extension */
#endif
    STANDARD_MODULE_PROPERTIES
};
/* }}} */

#ifdef COMPILE_DL_HUFFMAN
ZEND_GET_MODULE(huffman)
#endif


/* {{{ PHP_MINIT_FUNCTION
 */
PHP_MINIT_FUNCTION(huffman)
{
    REGISTER_STRING_CONSTANT("HUFFMAN_EXT_VERSION", HUFFMAN_VERSION, CONST_CS | CONST_PERSISTENT);
    
    return SUCCESS;
}
/* }}} */

/* {{{ PHP_MINFO_FUNCTION
 */
PHP_MINFO_FUNCTION(huffman)
{
    php_info_print_table_start();
    php_info_print_table_header(2, "huffman support", "enabled");
    php_info_print_table_end();
}
/* }}} */


/* {{{ proto string huffman_encode(string text)
   Encode text using Huffman algorithm */
PHP_FUNCTION(huffman_encode)
{
    unsigned char *text = NULL;
    int argc = ZEND_NUM_ARGS();
    unsigned int text_len;

    unsigned char *result = NULL;
    unsigned int result_len = 0;

    if (zend_parse_parameters(argc TSRMLS_CC, "s", &text, &text_len) == FAILURE) {
        RETURN_FALSE;
    }
 
    /* Note: lib returns 0 on success, 1 on failure */
    if(_huffman_encode(text, text_len, &result, &result_len)) {
        RETVAL_FALSE;
    } else {
        RETVAL_STRINGL(result, result_len, TRUE);
    }
}
/* }}} */

/* {{{ proto string huffman_decode(string encoded)
   Decode encoded text using Huffman algorithm */
PHP_FUNCTION(huffman_decode)
{
    unsigned char *encoded = NULL;
    int argc = ZEND_NUM_ARGS();
    unsigned int encoded_len;

    unsigned char *result = NULL;
    unsigned int result_len = 0;
    
    if (zend_parse_parameters(argc TSRMLS_CC, "s", &encoded, &encoded_len) == FAILURE) {
        RETURN_FALSE;
    }

    /* Note: lib returns 0 on success, 1 on failure */
    if (_huffman_decode(encoded, encoded_len, &result, &result_len)) {
        RETVAL_FALSE;
    } else {
        RETVAL_STRINGL(result, result_len, TRUE);
    }
}
/* }}} */

/* {{{ proto boolean huffman_encode_file(string file_in, string file_out)
   Encode file using Huffman algorithm */
PHP_FUNCTION(huffman_encode_file)
{
    char *file_in  = NULL;
    char *file_out = NULL;    
    int argc = ZEND_NUM_ARGS();
    int file_in_len;
    int file_out_len;

    FILE *in  = NULL;
    FILE *out = NULL;
    
    if (zend_parse_parameters(argc TSRMLS_CC, "ss", &file_in, &file_in_len, &file_out, &file_out_len) == FAILURE) {
        RETURN_FALSE;
    }

    in = fopen(file_in, "rb");
    
    if (!in) {
        RETURN_FALSE;
    }
    
    out = fopen(file_out, "wb");
    
    if (!out) {
        RETURN_FALSE;
    }
    
    /* Note: lib returns 0 on success, 1 on failure */
    if (!_huffman_encode_file(in,out)) {
        RETURN_TRUE;
    } else {
        RETURN_FALSE;
    }
}
/* }}} */

/* {{{ proto boolean huffman_decode_file(string file_in, string file_out)
   Decode encoded file using Huffman algorithm */
PHP_FUNCTION(huffman_decode_file)
{
    char *file_in  = NULL;
    char *file_out = NULL;
    int argc = ZEND_NUM_ARGS();
    int file_in_len;
    int file_out_len;

    FILE *in  = NULL;
    FILE *out = NULL;
    
    if (zend_parse_parameters(argc TSRMLS_CC, "ss", &file_in, &file_in_len, &file_out, &file_out_len) == FAILURE) {
        RETURN_FALSE;
    }

    in = fopen(file_in, "rb");
    
    if (!in) {
        RETURN_FALSE;
    }
    
    out = fopen(file_out, "wb");
    
    if (!out) {
        RETURN_FALSE;
    }
    
    /* Note: lib returns 0 on success, 1 on failure */
    if (!_huffman_decode_file(in,out)) {
        RETURN_TRUE;
    } else {
        RETURN_FALSE;
    }
}
/* }}} */


/*
 * Local variables:
 * tab-width: 4
 * c-basic-offset: 4
 * End:
 * vim600: noet sw=4 ts=4 fdm=marker
 * vim<600: noet sw=4 ts=4
 */
